@@ -53,132 +53,13 @@ inline bool exists(const std::string &filename) {
     return in.good();
 }
 
-::vector<::string> get_tokens(const ::string &line) {
-    ::vector<::string> tokens;
-    size_t prev = 0, pos = 0;
-    ::string token;
-    // copied from https://stackoverflow.com/a/7621814
-    while ((pos = line.find_first_of(DELIMITER, prev))
-           != ::string::npos) {
-        if (pos > prev) {
-            tokens.emplace_back(line.substr(prev, pos - prev));
-        }
-        prev = pos + 1;
-    }
-    if (prev < line.length())
-        tokens.emplace_back(line.substr(prev, std::string::npos));
-    return tokens;
-}
+// implemented in thunder IO
+::vector<::string> get_tokens(const ::string &line);
 
+// implemented in thunder IO
 ::pair<::map<::string, ::vector<::pair<::string, ::string>>>,
        ::map<::string, uint32_t>>
-load_netlist(const std::string &filename) {
-    if (!::exists(filename))
-        throw ::runtime_error(filename + " does not exist");
-    ::ifstream in;
-    in.open(filename);
-
-    ::string line;
-    bool in_netlist = false;
-    bool in_bus = false;
-
-    ::map<::string, ::vector<::pair<::string, ::string>>> netlist;
-    ::map<::string, uint32_t> track_mode;
-
-    while(std::getline(in, line)) {
-        // we are only interested in the packed netlist section
-        trim(line);
-        if (line[0] == '#')
-            continue;
-        if (in_netlist) {
-            if (line.empty()) {
-                in_netlist = false;
-                continue;
-            }
-            const ::vector<::string> tokens = get_tokens(line);
-
-            if (tokens.size() % 2 != 1)
-                throw ::runtime_error("unable to process line " + line);
-            const ::string &net_id = tokens[0];
-            ::vector<::pair<::string, ::string>> net;
-            for (uint32_t i = 1; i < tokens.size(); i+= 2) {
-                ::string const &blk_id = tokens[i];
-                ::string const &port = tokens[i + 1];
-                net.emplace_back(make_pair(blk_id, port));
-            }
-            netlist.insert({net_id, net});
-            // skip the rest of logic
-            continue;
-        }
-
-        if (in_bus) {
-            if (line.empty()) {
-                in_bus = false;
-                continue;
-            }
-            const ::vector<::string> tokens = get_tokens(line);
-            if (tokens.size() != 2)
-                throw ::runtime_error("unable to process line " + line);
-            const ::string net_id = tokens[0];
-            const ::string track_str = tokens[1];
-
-            auto width = static_cast<uint32_t>(std::stoi(track_str));
-            track_mode.insert({net_id, width});
-
-            // skip the rest of logic
-            continue;
-        }
-
-        // state control
-        if (line == "Netlists:") {
-            in_netlist = true;
-            continue;
-        } else if (line == "Netlist Bus:") {
-            in_bus = true;
-        } else {
-            in_netlist = false;
-            in_bus = false;
-        }
-    }
-
-    if (netlist.size() != track_mode.size()) {
-        throw ::runtime_error("netlist size doesn't match with netlist bus");
-    }
-    return {netlist, track_mode};
-}
-
-
-std::map<std::string, std::pair<uint32_t, uint32_t>>
-load_placement(const std::string &filename) {
-    if (!::exists(filename))
-        throw ::runtime_error(filename + " does not exist");
-    ::ifstream in;
-    in.open(filename);
-
-    ::string line;
-    uint32_t line_num = 0;
-    std::map<std::string, std::pair<uint32_t, uint32_t>> placement;
-
-    while(std::getline(in, line)) {
-        if (line_num < 2) {
-            line_num++;
-            continue;
-        }
-        trim(line);
-
-        auto tokens = get_tokens(line);
-        if (tokens.size() != 4)
-            throw ::runtime_error("unable to process line " + line);
-        auto x = static_cast<uint32_t>(std::stoi(tokens[1]));
-        auto y = static_cast<uint32_t>(std::stoi(tokens[2]));
-        auto blk_id = tokens[3].substr(1, ::string::npos);
-
-        placement.insert({blk_id, {x, y}});
-        line_num++;
-    }
-
-    return placement;
-}
+load_netlist(const std::string &filename);
 
 // used to print identifiers for edges
 ::string node_to_string(const ::string &pad,
@@ -254,7 +135,7 @@ void print_sb(std::ofstream &out, const std::string &pad, const Switch &sb) {
         << sb.num_track << endl;
     out << BEGIN << endl;
     const auto wires = sb.internal_wires();
-    for (auto const iter : wires) {
+    for (auto const &iter : wires) {
         auto [track_from, side_from, track_to, side_to] = iter;
         out << pad << track_from << " " << gsv(side_from) << " "
             << track_to << " " << gsv(side_to) << endl;
@@ -323,6 +204,21 @@ void dump_routing_graph(RoutingGraph &graph,
     }
 }
 
+
+void dump_wave_info(const std::map<std::string, uint64_t> &wave_info, const std::string &path) {
+    // need to sort based on the pin id to get a consistent result
+    std::ofstream out;
+    out.open(path);
+    static const ::string PAD = "  ";
+    out << "BLK_ID" << PAD << "WAVE_NUM" << std::endl;
+
+    for (auto const &[pin, w]: wave_info) {
+        out << pin << PAD << wave_info.at(pin) << std::endl;
+    }
+    out.close();
+}
+
+
 inline uint32_t stou(const std::string &str) {
     return static_cast<uint32_t>(std::stoi(str));
 }
@@ -512,7 +408,7 @@ void dump_routing_result(const Router &r, const std::string &filename) {
 
     auto routes = r.realize();
     const auto &netlist = r.get_netlist();
-    for (const auto &net : netlist) {
+    for (const auto &[id, net] : netlist) {
         const auto &net_id = net.name;
         auto const segments = routes.at(net.name);
         out << "Net ID: " << net_id << " Segment Size: "
@@ -550,7 +446,6 @@ void setup_router_input(Router &r, const std::string &packed_filename,
                         const std::string &placement_filename,
                         uint32_t width) {
     auto [netlist, track_mode] = load_netlist(packed_filename);
-    printf("netlist: %ld\n", netlist.size());
     auto placement = load_placement(placement_filename);
     for (auto const &it : placement) {
         auto [x, y] = it.second;
